Module prelude

Module prelude 

This prelude module re-exports every SIMD trait defined in this library. It’s useful for accessing trait methods.

Only traits are exported through the prelude; types must be exported separately.

Traits

Bytes
Conversion of SIMD vectors to and from same-width vectors of u8 lanes.
Select
Element-wise selection between two SIMD vectors using self.
Simd
The main SIMD trait, implemented by all SIMD token types.
SimdBase
Base functionality implemented by all SIMD vectors.
SimdCombine
Concatenation of two SIMD vectors.
SimdCvtFloat
Construction of floating point vectors from integers
SimdCvtTruncate
Construction of integer vectors from floats by truncation
SimdElement
Types that can be used as elements in SIMD vectors.
SimdFloat
Functionality implemented by floating-point SIMD vectors.
SimdFloatElement
Types that can be used as elements in float SIMD vectors.
SimdFrom
Value conversion, adding a SIMD blessing.
SimdInt
Functionality implemented by (signed and unsigned) integer SIMD vectors.
SimdIntElement
Types that can be used as elements in integer SIMD vectors.
SimdInterleaved
Interleaved loads and stores for SIMD vectors.
SimdInto
Value conversion, adding a SIMD blessing.
SimdMask
Functionality implemented by SIMD masks.
SimdNarrow
Narrowing conversion of two numeric SIMD vectors.
SimdSplit
Splitting of one SIMD vector into two.
SimdWiden
Widening conversion of a numeric SIMD vector.