Avx2

Struct Avx2 

pub struct Avx2 { /* private fields */ }
Available on x86 or x86-64 only.
A token for AVX2 intrinsics on x86 and x86_64, representing the x86-64-v3 level.

Browsing the documentation

The method list on this struct is very verbose. A better way to browse the docs is by looking at vector types such as u32x4, f32x4 or mask32x4. They include all the operations listed here, and also provide some additional convenience methods.

Implementations

Source

impl Avx2

pub const fn assume_supported() -> Self

Available on target features avx2 and bmi1 and bmi2 and cmpxchg16b and f16c and fma and fxsr and lzcnt and movbe and popcnt and xsave only.

Create a SIMD token proving that the x86-64-v3 features are available.

Most users should safely obtain the token from Level::new instead of calling this function.

This function can be called without an unsafe block from a function with all the required target features enabled via the #[target_feature] annotation.

Safety

When invoking this function through an unsafe block, the caller must ensure that the current CPU supports avx2, bmi1, bmi2, cmpxchg16b, f16c, fma, fxsr, lzcnt, movbe, popcnt, xsave.