pub struct u32x4<S: Simd> {
pub simd: S,
/* private fields */
}Expand description
A SIMD vector of 4 u32 elements.
You may construct this vector type using the Self::splat, Self::from_slice, Self::simd_from, Self::from_fn, and Self::block_splat methods.
fn construct_simd<S: Simd>(simd: S) {
// From a single scalar value:
let a = u32x4::splat(simd, 1);
let b = u32x4::simd_from(simd, 1);
// From a slice:
let c = u32x4::from_slice(simd, &[1, 2, 3, 4]);
// From an array:
let d = u32x4::simd_from(simd, [1, 2, 3, 4]);
// From an element-wise function:
let e = u32x4::from_fn(simd, |i| i as u32);
}Fields
simd: STrait Implementations
Sourceimpl<S: Simd> AddAssign for u32x4<S>
impl<S: Simd> AddAssign for u32x4<S>
Sourcefn add_assign(&mut self, rhs: Self)
fn add_assign(&mut self, rhs: Self)
Add two vectors element-wise, wrapping on overflow.
Sourceimpl<S: Simd> AddAssign<u32> for u32x4<S>
impl<S: Simd> AddAssign<u32> for u32x4<S>
Sourcefn add_assign(&mut self, rhs: u32)
fn add_assign(&mut self, rhs: u32)
+= operation. Read moreSourceimpl<S: Simd> BitAndAssign for u32x4<S>
impl<S: Simd> BitAndAssign for u32x4<S>
Sourcefn bitand_assign(&mut self, rhs: Self)
fn bitand_assign(&mut self, rhs: Self)
Compute the bitwise AND of two vectors.
Sourceimpl<S: Simd> BitAndAssign<u32> for u32x4<S>
impl<S: Simd> BitAndAssign<u32> for u32x4<S>
Sourcefn bitand_assign(&mut self, rhs: u32)
fn bitand_assign(&mut self, rhs: u32)
&= operation. Read moreSourceimpl<S: Simd> BitOrAssign for u32x4<S>
impl<S: Simd> BitOrAssign for u32x4<S>
Sourcefn bitor_assign(&mut self, rhs: Self)
fn bitor_assign(&mut self, rhs: Self)
Compute the bitwise OR of two vectors.
Sourceimpl<S: Simd> BitOrAssign<u32> for u32x4<S>
impl<S: Simd> BitOrAssign<u32> for u32x4<S>
Sourcefn bitor_assign(&mut self, rhs: u32)
fn bitor_assign(&mut self, rhs: u32)
|= operation. Read moreSourceimpl<S: Simd> BitXorAssign for u32x4<S>
impl<S: Simd> BitXorAssign for u32x4<S>
Sourcefn bitxor_assign(&mut self, rhs: Self)
fn bitxor_assign(&mut self, rhs: Self)
Compute the bitwise XOR of two vectors.
Sourceimpl<S: Simd> BitXorAssign<u32> for u32x4<S>
impl<S: Simd> BitXorAssign<u32> for u32x4<S>
Sourcefn bitxor_assign(&mut self, rhs: u32)
fn bitxor_assign(&mut self, rhs: u32)
^= operation. Read moreSourceimpl<S: Simd> Bytes for u32x4<S>
impl<S: Simd> Bytes for u32x4<S>
impl<S: Copy + Simd> Copy for u32x4<S>where
S::u32x4: Copy,
Sourceimpl<S: Simd> MulAssign for u32x4<S>
impl<S: Simd> MulAssign for u32x4<S>
Sourcefn mul_assign(&mut self, rhs: Self)
fn mul_assign(&mut self, rhs: Self)
Multiply two vectors element-wise, wrapping on overflow.
Sourceimpl<S: Simd> MulAssign<u32> for u32x4<S>
impl<S: Simd> MulAssign<u32> for u32x4<S>
Sourcefn mul_assign(&mut self, rhs: u32)
fn mul_assign(&mut self, rhs: u32)
*= operation. Read moreSourceimpl<S: Simd> Shl for u32x4<S>
impl<S: Simd> Shl for u32x4<S>
Sourcefn shl(self, rhs: Self) -> Self::Output
fn shl(self, rhs: Self) -> Self::Output
Shift each element left by the given number of bits.
Bits shifted out of the left side are discarded, and zeros are shifted in on the right.
When shifting out of bounds (e.g. shifting a 32-bit value by 32 or more), the result is implementation-defined and may vary by platform.
This operation is not implemented in hardware on all platforms. On WebAssembly, and on x86 platforms without AVX2, this will use a fallback scalar implementation.
Sourceimpl<S: Simd> Shl<u32> for u32x4<S>
impl<S: Simd> Shl<u32> for u32x4<S>
Sourcefn shl(self, rhs: u32) -> Self::Output
fn shl(self, rhs: u32) -> Self::Output
Shift each element left by the given number of bits.
Bits shifted out of the left side are discarded, and zeros are shifted in on the right.
When shifting out of bounds (e.g. shifting a 32-bit value by 32 or more), the result is implementation-defined and may vary by platform.
Sourceimpl<S: Simd> ShlAssign for u32x4<S>
impl<S: Simd> ShlAssign for u32x4<S>
Sourcefn shl_assign(&mut self, rhs: Self)
fn shl_assign(&mut self, rhs: Self)
Shift each element left by the given number of bits.
Bits shifted out of the left side are discarded, and zeros are shifted in on the right.
When shifting out of bounds (e.g. shifting a 32-bit value by 32 or more), the result is implementation-defined and may vary by platform.
This operation is not implemented in hardware on all platforms. On WebAssembly, and on x86 platforms without AVX2, this will use a fallback scalar implementation.
Sourceimpl<S: Simd> ShlAssign<u32> for u32x4<S>
impl<S: Simd> ShlAssign<u32> for u32x4<S>
Sourcefn shl_assign(&mut self, rhs: u32)
fn shl_assign(&mut self, rhs: u32)
<<= operation. Read moreSourceimpl<S: Simd> Shr for u32x4<S>
impl<S: Simd> Shr for u32x4<S>
Sourcefn shr(self, rhs: Self) -> Self::Output
fn shr(self, rhs: Self) -> Self::Output
Shift each element right by the corresponding element in another vector.
For unsigned integers, zeros are shifted in on the left. For signed integers, the sign bit is replicated.
When shifting out of bounds (e.g. shifting a 32-bit value by 32 or more), the result is implementation-defined and may vary by platform.
This operation is not implemented in hardware on all platforms. On WebAssembly, and on x86 platforms without AVX2, this will use a fallback scalar implementation.
Sourceimpl<S: Simd> Shr<u32> for u32x4<S>
impl<S: Simd> Shr<u32> for u32x4<S>
Sourcefn shr(self, rhs: u32) -> Self::Output
fn shr(self, rhs: u32) -> Self::Output
Shift each element right by the given number of bits.
For unsigned integers, zeros are shifted in on the left. For signed integers, the sign bit is replicated.
When shifting out of bounds (e.g. shifting a 32-bit value by 32 or more), the result is implementation-defined and may vary by platform.
Sourceimpl<S: Simd> ShrAssign for u32x4<S>
impl<S: Simd> ShrAssign for u32x4<S>
Sourcefn shr_assign(&mut self, rhs: Self)
fn shr_assign(&mut self, rhs: Self)
Shift each element right by the corresponding element in another vector.
For unsigned integers, zeros are shifted in on the left. For signed integers, the sign bit is replicated.
When shifting out of bounds (e.g. shifting a 32-bit value by 32 or more), the result is implementation-defined and may vary by platform.
This operation is not implemented in hardware on all platforms. On WebAssembly, and on x86 platforms without AVX2, this will use a fallback scalar implementation.
Sourceimpl<S: Simd> ShrAssign<u32> for u32x4<S>
impl<S: Simd> ShrAssign<u32> for u32x4<S>
Sourcefn shr_assign(&mut self, rhs: u32)
fn shr_assign(&mut self, rhs: u32)
>>= operation. Read moreSourceimpl<S: Simd> SimdBase<S> for u32x4<S>
impl<S: Simd> SimdBase<S> for u32x4<S>
Sourceconst N: usize = 4
const N: usize = 4
Simd::f32s) and
want to process data in native-width chunks.Sourcetype ByteVector = u8x16<S>
type ByteVector = u8x16<S>
u8 lanes used as the byte representation. Read moreSourcetype Array = [u32; 4]
type Array = [u32; 4]
[Self::Element; Self::N]. It has the same layout as
this vector type, but likely has a lower alignment.fn as_slice(&self) -> &[u32]
fn as_mut_slice(&mut self) -> &mut [u32]
Sourcefn from_slice(simd: S, slice: &[u32]) -> Self
fn from_slice(simd: S, slice: &[u32]) -> Self
Sourcefn store_slice(&self, slice: &mut [u32])
fn store_slice(&self, slice: &mut [u32])
Sourcefn splat(simd: S, val: u32) -> Self
fn splat(simd: S, val: u32) -> Self
Sourcefn block_splat(block: Self::Block) -> Self
fn block_splat(block: Self::Block) -> Self
Sourcefn from_fn(simd: S, f: impl FnMut(usize) -> u32) -> Self
fn from_fn(simd: S, f: impl FnMut(usize) -> u32) -> Self
f with that element’s lane index (from 0 to
SimdBase::N - 1).Sourcefn slide_within_blocks<const SHIFT: usize>(
self,
rhs: impl SimdInto<Self, S>,
) -> Self
fn slide_within_blocks<const SHIFT: usize>( self, rhs: impl SimdInto<Self, S>, ) -> Self
slide, but operates independently on each 128-bit block.Sourcefn swizzle_dyn_within_blocks(
self,
indices: impl SimdInto<Self::Bytes, S>,
) -> Self
fn swizzle_dyn_within_blocks( self, indices: impl SimdInto<Self::Bytes, S>, ) -> Self
Sourcefn swizzle_dyn(self, indices: impl SimdInto<Self::Bytes, S>) -> Self
fn swizzle_dyn(self, indices: impl SimdInto<Self::Bytes, S>) -> Self
Sourcefn swizzle_dyn_precise(self, indices: impl SimdInto<Self::Bytes, S>) -> Self
fn swizzle_dyn_precise(self, indices: impl SimdInto<Self::Bytes, S>) -> Self
Sourcefn max(self, rhs: impl SimdInto<Self, S>) -> Self
fn max(self, rhs: impl SimdInto<Self, S>) -> Self
Sourcefn min(self, rhs: impl SimdInto<Self, S>) -> Self
fn min(self, rhs: impl SimdInto<Self, S>) -> Self
Sourcefn max_precise(self, rhs: impl SimdInto<Self, S>) -> Self
fn max_precise(self, rhs: impl SimdInto<Self, S>) -> Self
Sourcefn min_precise(self, rhs: impl SimdInto<Self, S>) -> Self
fn min_precise(self, rhs: impl SimdInto<Self, S>) -> Self
Sourcefn simd_eq(self, rhs: impl SimdInto<Self, S>) -> Self::Mask
fn simd_eq(self, rhs: impl SimdInto<Self, S>) -> Self::Mask
Sourcefn simd_lt(self, rhs: impl SimdInto<Self, S>) -> Self::Mask
fn simd_lt(self, rhs: impl SimdInto<Self, S>) -> Self::Mask
Sourcefn simd_le(self, rhs: impl SimdInto<Self, S>) -> Self::Mask
fn simd_le(self, rhs: impl SimdInto<Self, S>) -> Self::Mask
Sourcefn simd_ge(self, rhs: impl SimdInto<Self, S>) -> Self::Mask
fn simd_ge(self, rhs: impl SimdInto<Self, S>) -> Self::Mask
Sourcefn simd_gt(self, rhs: impl SimdInto<Self, S>) -> Self::Mask
fn simd_gt(self, rhs: impl SimdInto<Self, S>) -> Self::Mask
Sourcefn zip_low(self, rhs: impl SimdInto<Self, S>) -> Self
fn zip_low(self, rhs: impl SimdInto<Self, S>) -> Self
Sourcefn zip_high(self, rhs: impl SimdInto<Self, S>) -> Self
fn zip_high(self, rhs: impl SimdInto<Self, S>) -> Self
Sourcefn unzip_low(self, rhs: impl SimdInto<Self, S>) -> Self
fn unzip_low(self, rhs: impl SimdInto<Self, S>) -> Self
Sourcefn unzip_high(self, rhs: impl SimdInto<Self, S>) -> Self
fn unzip_high(self, rhs: impl SimdInto<Self, S>) -> Self
Sourcefn interleave(self, rhs: impl SimdInto<Self, S>) -> (Self, Self)
fn interleave(self, rhs: impl SimdInto<Self, S>) -> (Self, Self)
Sourcefn deinterleave(self, rhs: impl SimdInto<Self, S>) -> (Self, Self)
fn deinterleave(self, rhs: impl SimdInto<Self, S>) -> (Self, Self)
Sourcefn load_array(simd: S, val: Self::Array) -> Self
fn load_array(simd: S, val: Self::Array) -> Self
Sourcefn load_array_ref(simd: S, val: &Self::Array) -> Self
fn load_array_ref(simd: S, val: &Self::Array) -> Self
Sourcefn as_array_ref(&self) -> &Self::Array
fn as_array_ref(&self) -> &Self::Array
Sourcefn as_array_mut(&mut self) -> &mut Self::Array
fn as_array_mut(&mut self) -> &mut Self::Array
Sourcefn store_array(self, dest: &mut Self::Array)
fn store_array(self, dest: &mut Self::Array)
Sourcefn rotate_elements_left<const OFFSET: usize>(self) -> Self
fn rotate_elements_left<const OFFSET: usize>(self) -> Self
OFFSET. Read moreSourcefn rotate_elements_right<const OFFSET: usize>(self) -> Self
fn rotate_elements_right<const OFFSET: usize>(self) -> Self
OFFSET. Read moreSourcefn shift_elements_left<const OFFSET: usize>(
self,
padding: Self::Element,
) -> Self
fn shift_elements_left<const OFFSET: usize>( self, padding: Self::Element, ) -> Self
Sourceimpl<S: Simd> SimdCombine<S> for u32x4<S>
impl<S: Simd> SimdCombine<S> for u32x4<S>
Sourceimpl<S: Simd> SimdCvtFloat<u32x4<S>> for f32x4<S>
impl<S: Simd> SimdCvtFloat<u32x4<S>> for f32x4<S>
Sourcefn float_from(x: u32x4<S>) -> Self
fn float_from(x: u32x4<S>) -> Self
Convert each unsigned 32-bit integer element to a floating-point value.
Values that cannot be exactly represented are rounded to the nearest representable value.
Sourceimpl<S: Simd> SimdCvtTruncate<f32x4<S>> for u32x4<S>
impl<S: Simd> SimdCvtTruncate<f32x4<S>> for u32x4<S>
Sourcefn truncate_from(x: f32x4<S>) -> Self
fn truncate_from(x: f32x4<S>) -> Self
Convert each floating-point element to an unsigned 32-bit integer, truncating towards zero.
Out-of-range values or NaN will produce implementation-defined results.
On x86 platforms below AVX-512, this operation will still be slower than converting to i32, because there is no native instruction for converting to u32.
If you know your values fit within range of an i32, you should convert to an i32 and cast to your desired datatype afterwards.
Sourcefn truncate_from_precise(x: f32x4<S>) -> Self
fn truncate_from_precise(x: f32x4<S>) -> Self
Convert each floating-point element to an unsigned 32-bit integer, truncating towards zero.
Out-of-range values are saturated to the closest in-range value. NaN becomes 0.
Sourceimpl<S: Simd> SimdInt<S> for u32x4<S>
impl<S: Simd> SimdInt<S> for u32x4<S>
Sourcefn to_float<T: SimdCvtFloat<Self>>(self) -> T
fn to_float<T: SimdCvtFloat<Self>>(self) -> T
SimdCvtFloat::float_from, and can only be called if there
actually exists a target type of the same bit width (f32 or f64).