pub trait SimdElement: Seal {
type Mask: SimdElement;
}Expand description
Types that can be used as elements in SIMD vectors.
Required Associated Types
Sourcetype Mask: SimdElement
type Mask: SimdElement
The associated mask lane type. This will be a signed integer of the same size as this type.
Dyn Compatibility
This trait is dyn compatible.
In older versions of Rust, dyn compatibility was called "object safety".