SimdInt

Trait SimdInt 

pub trait SimdInt<S: Simd>:
    SimdBase<S, Element: SimdIntElement>
    + Seal
    + BitAnd<Output = Self>
    + BitAndAssign
    + BitAnd<Self::Element, Output = Self>
    + BitAndAssign<Self::Element>
    + BitOr<Output = Self>
    + BitOrAssign
    + BitOr<Self::Element, Output = Self>
    + BitOrAssign<Self::Element>
    + BitXor<Output = Self>
    + BitXorAssign
    + BitXor<Self::Element, Output = Self>
    + BitXorAssign<Self::Element>
    + Not<Output = Self>
    + Shl<u32, Output = Self>
    + ShlAssign<u32>
    + Shl<Output = Self>
    + ShlAssign
    + Shr<u32, Output = Self>
    + ShrAssign<u32>
    + Shr<Output = Self>
    + ShrAssign {
    // Provided method
    fn to_float<T: SimdCvtFloat<Self>>(self) -> T { ... }
}
Functionality implemented by (signed and unsigned) integer SIMD vectors.

Provided Methods

fn to_float<T: SimdCvtFloat<Self>>(self) -> T

Convert this integer type to a floating-point type. This is a convenience method that delegates to SimdCvtFloat::float_from, and can only be called if there actually exists a target type of the same bit width (f32 or f64).

Dyn Compatibility

This trait is not dyn compatible.

In older versions of Rust, dyn compatibility was called "object safety".

Implementors

Source

impl<S: Simd> SimdInt<S> for i8x16<S>

Source

impl<S: Simd> SimdInt<S> for i8x32<S>

Source

impl<S: Simd> SimdInt<S> for i8x64<S>

Source

impl<S: Simd> SimdInt<S> for i16x8<S>

Source

impl<S: Simd> SimdInt<S> for i16x16<S>

Source

impl<S: Simd> SimdInt<S> for i16x32<S>

Source

impl<S: Simd> SimdInt<S> for i32x4<S>

Source

impl<S: Simd> SimdInt<S> for i32x8<S>

Source

impl<S: Simd> SimdInt<S> for i32x16<S>

Source

impl<S: Simd> SimdInt<S> for i64x2<S>

Source

impl<S: Simd> SimdInt<S> for i64x4<S>

Source

impl<S: Simd> SimdInt<S> for i64x8<S>

Source

impl<S: Simd> SimdInt<S> for u8x16<S>

Source

impl<S: Simd> SimdInt<S> for u8x32<S>

Source

impl<S: Simd> SimdInt<S> for u8x64<S>

Source

impl<S: Simd> SimdInt<S> for u16x8<S>

Source

impl<S: Simd> SimdInt<S> for u16x16<S>

Source

impl<S: Simd> SimdInt<S> for u16x32<S>

Source

impl<S: Simd> SimdInt<S> for u32x4<S>

Source

impl<S: Simd> SimdInt<S> for u32x8<S>

Source

impl<S: Simd> SimdInt<S> for u32x16<S>

Source

impl<S: Simd> SimdInt<S> for u64x2<S>

Source

impl<S: Simd> SimdInt<S> for u64x4<S>

Source

impl<S: Simd> SimdInt<S> for u64x8<S>