pub trait SimdFloatElement:
SimdElement
+ Neg<Output = Self>
+ From<i8>
+ From<u8> { }Expand description
Types that can be used as elements in float SIMD vectors.
The scalar conversion bounds are limited to types that every floating-point element can represent losslessly, including f16 for forward-compatibility.
Dyn Compatibility
This trait is not dyn compatible.
In older versions of Rust, dyn compatibility was called "object safety".